@@ -41,6 +41,9 @@
 
 #include <id3v1genres.h>
 
+#include <array>
+#include <utility>
+
 #include "actioncollection.h"
 #include "collectionlist.h"
 #include "iconsupport.h"
@@ -51,6 +54,8 @@
 
 #undef KeyRelease
 
+using std::as_const;
+
 class FileNameValidator final : public QValidator
 {
 public:
@@ -160,11 +165,11 @@ void TagEditor::slotSetItems(const PlaylistItemList &list)
     // Store the playlist that we're setting because saveChangesPrompt
     // can delete the PlaylistItems in list.
 
-    Playlist *itemPlaylist = 0;
+    Playlist *itemPlaylist = nullptr;
     if(!list.isEmpty())
         itemPlaylist = list.first()->playlist();
 
-    bool hadPlaylist = m_currentPlaylist != 0;
+    bool hadPlaylist = m_currentPlaylist != nullptr;
 
     saveChangesPrompt();
 
@@ -174,7 +179,7 @@ void TagEditor::slotSetItems(const PlaylistItemList &list)
     }
 
     if((hadPlaylist && !m_currentPlaylist) || !itemPlaylist) {
-        m_currentPlaylist = 0;
+        m_currentPlaylist = nullptr;
         m_items.clear();
     }
     else {
@@ -244,104 +249,88 @@ void TagEditor::slotRefresh()
 
     commentBox->setPlainText(tag->comment());
 
-    // Start at the second item, since we've already processed the first.
+    // If there is more than one item in the items that we're dealing with then add
+    // 'Enable' boxes to use to determine whether to modify tags en masse or not
 
-    PlaylistItemList::Iterator it = m_items.begin();
-    ++it;
+    if(m_items.count() > 1) {
+        const std::array<QWidget*, 6> inputs {
+            fileNameLabel, fileNameBox, lengthLabel, lengthBox, bitrateLabel, bitrateBox
+        };
 
-    // If there is more than one item in the m_items that we're dealing with...
-
-    QList<QWidget *> disabledForMulti;
-
-    disabledForMulti << fileNameLabel << fileNameBox << lengthLabel << lengthBox
-                     << bitrateLabel << bitrateBox;
-
-    foreach(QWidget *w, disabledForMulti) {
-        w->setDisabled(m_items.size() > 1);
-        if(m_items.size() > 1 && !w->inherits("QLabel"))
-            QMetaObject::invokeMethod(w, "clear");
+        for(auto w : inputs) {
+            w->setDisabled(true);
+            if(!w->inherits("QLabel"))
+                QMetaObject::invokeMethod(w, "clear");
+        }
     }
 
-    if(it != m_items.end()) {
-
-        foreach(QCheckBox *box, m_enableBoxes) {
-            box->setChecked(true);
+    for(QCheckBox *box : as_const(m_enableBoxes)) {
+        box->setChecked(true);
+        if(m_items.count() > 1)
             box->show();
-        }
-
-        // Yep, this is ugly.  Loop through all of the files checking to see
-        // if their fields are the same.  If so, by default, enable their
-        // checkbox.
-
-        // Also, if there are more than 50 m_items, don't scan all of them.
-
-        if(m_items.count() > 50) {
-            m_enableBoxes[artistNameBox]->setChecked(false);
-            m_enableBoxes[trackNameBox]->setChecked(false);
-            m_enableBoxes[albumNameBox]->setChecked(false);
-            m_enableBoxes[genreBox]->setChecked(false);
-            m_enableBoxes[trackSpin]->setChecked(false);
-            m_enableBoxes[yearSpin]->setChecked(false);
-            m_enableBoxes[commentBox]->setChecked(false);
-        }
-        else {
-            for(; it != m_items.end(); ++it) {
-                tag = (*it)->file().tag();
-
-                if(tag) {
-
-                    if(artistNameBox->currentText() != tag->artist() &&
-                       m_enableBoxes.contains(artistNameBox))
-                    {
-                        artistNameBox->lineEdit()->clear();
-                        m_enableBoxes[artistNameBox]->setChecked(false);
-                    }
-                    if(trackNameBox->text() != tag->title() &&
-                       m_enableBoxes.contains(trackNameBox))
-                    {
-                        trackNameBox->clear();
-                        m_enableBoxes[trackNameBox]->setChecked(false);
-                    }
-                    if(albumNameBox->currentText() != tag->album() &&
-                       m_enableBoxes.contains(albumNameBox))
-                    {
-                        albumNameBox->lineEdit()->clear();
-                        m_enableBoxes[albumNameBox]->setChecked(false);
-                    }
-                    if(genreBox->currentText() != tag->genre() &&
-                       m_enableBoxes.contains(genreBox))
-                    {
-                        genreBox->lineEdit()->clear();
-                        m_enableBoxes[genreBox]->setChecked(false);
-                    }
-                    if(trackSpin->value() != tag->track() &&
-                       m_enableBoxes.contains(trackSpin))
-                    {
-                        trackSpin->setValue(0);
-                        m_enableBoxes[trackSpin]->setChecked(false);
-                    }
-                    if(yearSpin->value() != tag->year() &&
-                       m_enableBoxes.contains(yearSpin))
-                    {
-                        yearSpin->setValue(0);
-                        m_enableBoxes[yearSpin]->setChecked(false);
-                    }
-                    if(commentBox->toPlainText() != tag->comment() &&
-                       m_enableBoxes.contains(commentBox))
-                    {
-                        commentBox->clear();
-                        m_enableBoxes[commentBox]->setChecked(false);
-                    }
-                }
-            }
-        }
-    }
-    else {
-        foreach(QCheckBox *box, m_enableBoxes) {
-            box->setChecked(true);
+        else
             box->hide();
+    }
+
+    // Yep, this is ugly.  Loop through all of the files checking to see
+    // if their fields are the same.  If not, disable their checkbox.
+    // But skip the check if there are more than 50 items
+
+    bool enable_artist  = m_items.count() <= 50;
+    bool enable_title   = enable_artist;
+    bool enable_album   = enable_artist;
+    bool enable_genre   = enable_artist;
+    bool enable_track   = enable_artist;
+    bool enable_year    = enable_artist;
+    bool enable_comment = enable_artist;
+
+    if(m_items.count() <= 50) {
+        for(const PlaylistItem *item : as_const(m_items)) {
+            const auto tag = item->file().tag();
+
+            if(!tag) {
+                continue;
+            }
+
+            enable_artist  = enable_artist  && artistNameBox->currentText() == tag->artist();
+            enable_title   = enable_title   && trackNameBox->text() == tag->title();
+            enable_album   = enable_album   && albumNameBox->currentText() == tag->album();
+            enable_genre   = enable_genre   && genreBox->currentText() == tag->genre();
+            enable_track   = enable_track   && trackSpin->value() == tag->track();
+            enable_year    = enable_year    && yearSpin->value() == tag->year();
+            enable_comment = enable_comment && commentBox->toPlainText() == tag->comment();
         }
     }
+
+    if(!enable_artist && m_enableBoxes.contains(artistNameBox)) {
+        artistNameBox->lineEdit()->clear();
+        m_enableBoxes[artistNameBox]->setChecked(false);
+    }
+    if(!enable_title && m_enableBoxes.contains(trackNameBox)) {
+        trackNameBox->clear();
+        m_enableBoxes[trackNameBox]->setChecked(false);
+    }
+    if(!enable_album && m_enableBoxes.contains(albumNameBox)) {
+        albumNameBox->lineEdit()->clear();
+        m_enableBoxes[albumNameBox]->setChecked(false);
+    }
+    if(!enable_genre && m_enableBoxes.contains(genreBox)) {
+        genreBox->lineEdit()->clear();
+        m_enableBoxes[genreBox]->setChecked(false);
+    }
+    if(!enable_track && m_enableBoxes.contains(trackSpin)) {
+        trackSpin->setValue(0);
+        m_enableBoxes[trackSpin]->setChecked(false);
+    }
+    if(!enable_year && m_enableBoxes.contains(yearSpin)) {
+        yearSpin->setValue(0);
+        m_enableBoxes[yearSpin]->setChecked(false);
+    }
+    if(!enable_comment && m_enableBoxes.contains(commentBox)) {
+        commentBox->clear();
+        m_enableBoxes[commentBox]->setChecked(false);
+    }
+
     m_dataChanged = false;
 }
 
@@ -395,13 +384,14 @@ void TagEditor::updateCollection()
 
     m_genreList = list->uniqueSet(CollectionList::Genres);
 
-    foreach(const QString &genre, m_genreList)
+    for(const QString &genre : as_const(m_genreList)) {
         genreHash.insert(genre);
+    }
 
-    TagLib::StringList genres = TagLib::ID3v1::genreList();
-
-    for(TagLib::StringList::Iterator it = genres.begin(); it != genres.end(); ++it)
-        genreHash.insert(TStringToQString((*it)));
+    const auto genres = TagLib::ID3v1::genreList();
+    for(const auto &taglibGenre : genres) {
+        genreHash.insert(TStringToQString(taglibGenre));
+    }
 
     m_genreList = genreHash.values();
     m_genreList.sort();
@@ -435,10 +425,11 @@ void TagEditor::readConfig()
     ActionCollection::action<KToggleAction>("showEditor")->setChecked(show);
     setVisible(show);
 
-    TagLib::StringList genres = TagLib::ID3v1::genreList();
+    const auto genres = TagLib::ID3v1::genreList();
 
-    for(TagLib::StringList::ConstIterator it = genres.begin(); it != genres.end(); ++it)
-        m_genreList.append(TStringToQString((*it)));
+    for(const auto &taglibGenre : genres) {
+        m_genreList.append(TStringToQString(taglibGenre));
+    }
     m_genreList.sort();
 
     genreBox->clear();
@@ -459,7 +450,7 @@ void TagEditor::saveConfig()
 {
     // combo box completion modes
 
-    KConfigGroup config(KSharedConfig::openConfig(), "TagEditor");
+    KConfigGroup config(KSharedConfig::openConfig(), QStringLiteral("TagEditor"));
 
     if(artistNameBox && albumNameBox) {
         config.writeEntry("ArtistNameBoxMode", (int)artistNameBox->completionMode());
@@ -481,7 +472,7 @@ void TagEditor::setupActions()
     QAction *act = new QAction("document-save"_icon, i18n("&Save"), this);
     ActionCollection::actions()->addAction("saveItem", act);
     ActionCollection::actions()->setDefaultShortcut(act,
-            QKeySequence(Qt::CTRL + Qt::Key_T));
+            QKeySequence(Qt::CTRL | Qt::Key_T));
     connect(act, &QAction::triggered, this, &TagEditor::slotSave);
 }
 
@@ -522,77 +513,63 @@ void TagEditor::setupLayout()
 
 void TagEditor::save(const PlaylistItemList &list)
 {
-    if(!list.isEmpty() && m_dataChanged) {
-
-        QApplication::setOverrideCursor(Qt::WaitCursor);
-        m_dataChanged = false;
-        m_performingSave = true;
-
-        // The list variable can become corrupted if the playlist holding its
-        // items dies, which is possible as we edit tags.  So we need to copy
-        // the end marker.
-
-        PlaylistItemList::ConstIterator end = list.end();
-
-        for(PlaylistItemList::ConstIterator it = list.begin(); it != end; /* Deliberately missing */ ) {
-
-            // Process items before we being modifying tags, as the dynamic
-            // playlists will try to modify the file we edit if the tag changes
-            // due to our alterations here.
-
-            qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
-
-            PlaylistItem *item = *it;
-
-            // The playlist can be deleted from under us if this is the last
-            // item and we edit it so that it doesn't match the search, which
-            // means we can't increment the iterator, so let's do it now.
-
-            ++it;
-
-            QString fileName = item->file().fileInfo().path() + QDir::separator() +
-                               fileNameBox->text();
-            if(list.count() > 1)
-                fileName = item->file().fileInfo().absoluteFilePath();
-
-            Tag *tag = TagTransactionManager::duplicateTag(item->file().tag(), fileName);
-
-            // A bit more ugliness.  If there are multiple files that are
-            // being modified, they each have a "enabled" checkbox that
-            // says if that field is to be respected for the multiple
-            // files.  We have to check to see if that is enabled before
-            // each field that we write.
-
-            if(m_enableBoxes[artistNameBox]->isChecked())
-                tag->setArtist(artistNameBox->currentText());
-            if(m_enableBoxes[trackNameBox]->isChecked())
-                tag->setTitle(trackNameBox->text());
-            if(m_enableBoxes[albumNameBox]->isChecked())
-                tag->setAlbum(albumNameBox->currentText());
-            if(m_enableBoxes[trackSpin]->isChecked()) {
-                if(trackSpin->text().isEmpty())
-                    trackSpin->setValue(0);
-                tag->setTrack(trackSpin->value());
-            }
-            if(m_enableBoxes[yearSpin]->isChecked()) {
-                if(yearSpin->text().isEmpty())
-                    yearSpin->setValue(0);
-                tag->setYear(yearSpin->value());
-            }
-            if(m_enableBoxes[commentBox]->isChecked())
-                tag->setComment(commentBox->toPlainText());
-
-            if(m_enableBoxes[genreBox]->isChecked())
-                tag->setGenre(genreBox->currentText());
-
-            TagTransactionManager::instance()->changeTagOnItem(item, tag);
-        }
-
-        TagTransactionManager::instance()->commit();
-        CollectionList::instance()->playlistItemsChanged();
-        m_performingSave = false;
-        QApplication::restoreOverrideCursor();
+    if(!m_dataChanged || list.isEmpty()) {
+        return;
     }
+
+    QApplication::setOverrideCursor(Qt::WaitCursor);
+    m_dataChanged = false;
+    m_performingSave = true;
+
+    // Generate a 'command list' of tag changes to apply to the files for the
+    // music tracks first, before any changes are made. This both enables undo
+    // support, and ensures the GUI isn't accidentally broken in e.g. dynamic
+    // playlists as playlist items are modified.
+
+    for(PlaylistItem *item : list) {
+        QString fileName = item->file().fileInfo().path() + QDir::separator() +
+                           fileNameBox->text();
+        if(list.count() > 1)
+            fileName = item->file().fileInfo().absoluteFilePath();
+
+        Tag *tag = TagTransactionManager::duplicateTag(item->file().tag(), fileName);
+
+        // A bit more ugliness.  If there are multiple files that are
+        // being modified, they each have a "enabled" checkbox that
+        // says if that field is to be respected for the multiple
+        // files.  We have to check to see if that is enabled before
+        // each field that we write.
+
+        if(m_enableBoxes[artistNameBox]->isChecked())
+            tag->setArtist(artistNameBox->currentText());
+        if(m_enableBoxes[trackNameBox]->isChecked())
+            tag->setTitle(trackNameBox->text());
+        if(m_enableBoxes[albumNameBox]->isChecked())
+            tag->setAlbum(albumNameBox->currentText());
+        if(m_enableBoxes[trackSpin]->isChecked()) {
+            if(trackSpin->text().isEmpty())
+                trackSpin->setValue(0);
+            tag->setTrack(trackSpin->value());
+        }
+        if(m_enableBoxes[yearSpin]->isChecked()) {
+            if(yearSpin->text().isEmpty())
+                yearSpin->setValue(0);
+            tag->setYear(yearSpin->value());
+        }
+        if(m_enableBoxes[commentBox]->isChecked())
+            tag->setComment(commentBox->toPlainText());
+
+        if(m_enableBoxes[genreBox]->isChecked())
+            tag->setGenre(genreBox->currentText());
+
+        TagTransactionManager::instance()->changeTagOnItem(item, tag);
+    }
+
+    TagTransactionManager::instance()->commit();
+    m_performingSave = false;
+    QApplication::restoreOverrideCursor();
+
+    CollectionList::instance()->playlistItemsChanged();
 }
 
 void TagEditor::saveChangesPrompt()
@@ -602,24 +579,11 @@ void TagEditor::saveChangesPrompt()
 
     QStringList files;
 
-    foreach(const PlaylistItem *item, m_items)
+    for(const PlaylistItem *item : as_const(m_items)) {
         files.append(item->file().absFilePath());
+    }
 
-    const auto questionFunc =
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
-        &KMessageBox::questionTwoActionsList;
-#else
-        &KMessageBox::questionYesNoList;
-#endif
-
-    const auto primaryResponse =
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
-        KMessageBox::PrimaryAction;
-#else
-        KMessageBox::Yes;
-#endif
-
-    const auto response = questionFunc(
+    const auto response = KMessageBox::questionTwoActionsList(
                 this,
                 i18n("Do you want to save your changes to:\n"),
                 files,
@@ -630,7 +594,7 @@ void TagEditor::saveChangesPrompt()
                 KMessageBox::Notify
             );
 
-    if(response == primaryResponse) {
+    if(response == KMessageBox::PrimaryAction) {
         save(m_items);
     }
 }
@@ -663,7 +627,7 @@ void TagEditor::slotItemRemoved(PlaylistItem *item)
 void TagEditor::slotPlaylistDestroyed(Playlist *p)
 {
     if(m_currentPlaylist == p) {
-        m_currentPlaylist = 0;
+        m_currentPlaylist = nullptr;
         slotSetItems(PlaylistItemList());
     }
 }
